// 
// Linbhand serial communication functions
// Time-stamp: "2007-05-17 18:38:10 sugiura"
// 
// Copyright (c) 2007, Komei Sugiura (komei.sugiura@gmail.com)
// All rights reserved.
// Distributed under the BSD License.
// (See accompanying file license.txt)
// 
// Inspired by Stephen Nestinger's chbhand
// Modified by WATANABE Naoki (voodil@yahoo.co.jp)
// 
// For more detail on serial communication commands, 
// refer to Grasper Control Language Protocol
// www.barretttechnology.com/robot/products/hand/D0AA7A.pdf
// 
// Any comments or suggestions are welcome.

#include "GCLserial.h"


GCLserial::GCLserial(string dev){
   m_dev = dev;
   InitSerial();
}


GCLserial::~GCLserial(){
   CloseSerial();
}


void GCLserial::InitSerial()
{
   struct termios options;
 
   m_fd = open(m_dev.c_str(), O_RDWR|O_NOCTTY|O_NDELAY);
   if(m_fd == -1){
      cerr << "could not open " << m_dev << endl;
      exit(-1);
   }

   int error = tcgetattr(m_fd,&options);
   if(error){
      printf("tcgetattr() failed with error %d.\n", errno);
      CloseSerial();
      exit(-1);
   }

   /* Using with RAW mode? */
   cfmakeraw(&options);
   
   cfsetispeed(&options,B9600);
   cfsetospeed(&options,B9600); /* 9600 bps */
   /* 8N1 */
   options.c_cflag &= ~PARENB; /* No parity */
   options.c_cflag &= ~CSTOPB; /* 1 stop bit  */
   options.c_cflag &= ~CSIZE;
   options.c_cflag |= CS8; /* 8bit */

   options.c_cflag |= (CLOCAL|CREAD);

//    options.c_lflag &= (~ICANON);

//    options.c_cc[VMIN] = 0;
   options.c_cc[VMIN] = 1;	// min 1 byte
   /* Timeout */
//    options.c_cc[VTIME] = 1;
   options.c_cc[VTIME] = 0;

   error = tcsetattr(m_fd,TCSANOW,&options);
   if(error){
      printf("tcsetattr() failed with error %d.\n", errno);
      CloseSerial();
      exit(-1);
   }

   fcntl(m_fd,F_SETFL,0);     

}


void GCLserial::CloseSerial()
{
   if(close(m_fd) != 0){
      cerr << "could not close " << m_dev << endl;
      exit(-1);
   }
}


int GCLserial::CommRead(char * buf)
{
   // ꥢǥХɤ߹ǡfeedbackʬĹ֤
   // feedbackȤbuf˽񤭹ޤ
   char message = 0;
   int len = -1;
   int i = 0;
   memset(buf, 0, sizeof(buf));

   while(1){
      len = read(m_fd, &message, 1);
      if(len < 0) {
	 printf("read() failed with error %d\n", errno);
	 CloseSerial();
	 exit(-1);
      }

      if( len == 0 ) break;

      if( message == '=' ){	/* supervisory modeνλʸ */
	 read(m_fd, &message, 2);	/* Ĥ"> "ɤ */
	 buf[i] = '\0';
	 break;
      }
      if( message == '*' ){	/* realtime modeνλʸ */
	 buf[i] = '\0';
	 break;
      }
      buf[i] = message;
      i++;			/* ʸɤ߹߽ */
   }
   buf[i] = '\0';

   return i;
}


int GCLserial::CommWrite(char * buf)
{
   int len = write(m_fd, buf, strlen(buf));
   if(len < 0){
      cerr << "could not write " << m_dev << endl;
      CloseSerial();
      exit(-1);
   }
   return len;
}


int GCLserial::CommWrite(string buf)
{
   return CommWrite(buf.c_str());
}


