// 
// Linbhand base functions
// BarrettHand C++-function Library for Linux
// 
// Time-stamp: "2007-05-17 18:38:24 sugiura"
// 
// Copyright (c) 2007, Komei Sugiura (komei.sugiura@gmail.com)
// All rights reserved.
// Distributed under the BSD License.
// (See accompanying file license.txt)
// 
// Inspired by Stephen Nestinger's chbhand
// 
// For more detail on serial communication commands, 
// refer to Grasper Control Language Protocol
// www.barretttechnology.com/robot/products/hand/D0AA7A.pdf
// 
// Any comments or suggestions are welcome.


#include "Linbhand_base.h"


Linbhand_base::Linbhand_base(string dev)
{
   m_dev = dev;
   m_scom = new GCLserial(m_dev);
   memset(m_readbuf, 0, sizeof(m_readbuf));
   memset(m_writebuf, 0, sizeof(m_writebuf));
   memset(m_RTmotors, 0, sizeof(m_RTmotors));
   memset(m_RTflags, 0, sizeof(m_RTflags));
   memset(m_RTvel, 0, sizeof(m_RTvel));
   memset(m_RTgain, 0, sizeof(m_RTgain));

   // flush serial buffer
   sprintf(m_writebuf, "\r");
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);
}


Linbhand_base::~Linbhand_base()
{
   delete m_scom;
}


int Linbhand_base::InitHand(char * motor)
{
   // input
   // 1 : Finger F1, 2 : Finger F2, 3 : Finger F3, 4 or S : Spread Motion
   // G : F1, F2, and F3
   // Empty string : all motors
   sprintf(m_writebuf, "%sHI\r", motor);
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);
   return 0;
}


int Linbhand_base::Open(char * motor)
{
   sprintf(m_writebuf, "%sO\r", motor);
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);
   return 0;
}


int Linbhand_base::Close(char * motor)
{
   sprintf(m_writebuf, "%sC\r", motor);
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);


   return 0;
}


int Linbhand_base::StepOpen(char * motor, int value)
{
   sprintf(m_writebuf, "%sIO %d\r", motor, value);
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);
   return 0;
}


int Linbhand_base::StepClose(char * motor, int value)
{
   sprintf(m_writebuf, "%sIC %d\r", motor, value);
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);
   return 0;
}


int Linbhand_base::GoToDefault(char * motor)
{
   sprintf(m_writebuf, "%sM\r", motor);
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);
   return 0;
}


int Linbhand_base::GoToHome()
{
   sprintf(m_writebuf, "HOME\r");
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);
   return 0;
}


int Linbhand_base::GoToPosition(char * motor, int value)
{
   sprintf(m_writebuf, "%sM %d\r", motor, value);
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);
   return 0;
}


int Linbhand_base::GoToDifferentPositions(int value1, int value2, int value3, int value4)
{
   int values[] = {value1, value2, value3, value4};
   int dps[] = {0,0,0,0};	// for storing DP(default position) 
   Get("1234", "DP", dps);

   for(int i=0;i<4;i++){
      sprintf(m_writebuf, "%dFSET DP %d\r", i+1, values[i]);
      m_scom->CommWrite(m_writebuf);
      m_scom->CommRead(m_readbuf);
   }

   sprintf(m_writebuf, "M\r");
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);

   // restore
   for(int i=0;i<4;i++){
      sprintf(m_writebuf, "%dFSET DP %d\r", i+1, dps[i]);
      m_scom->CommWrite(m_writebuf);
      m_scom->CommRead(m_readbuf);
   }

   return 0;
}


int Linbhand_base::Get(char * motor, char * parameter, int * result)
{
//  result˷̤
// Example:
//    static int res[4];		// static뤳
//    bh.Get("123", "P", res);
//    cout << res[0] << endl;

//    vector<int> out(0);
   sprintf(m_writebuf, "%sFGET %s\r", motor, parameter);
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);

   // ʲ3Ԥäưʤ()
   // m_readbufؤΥݥ?
   char b[1];
   strncpy(b, m_readbuf, 1);
   string c = b;

   string tmp = m_readbuf;
   tmp.erase( 0, tmp.find('\n')+1 );	// ̿ʬ

   for(int i=0;i<4;i++){
      // եޥåȤ"10 20 30\n"Τ褦ʷǤΤڤФ
      // vector
      result[i] = atoi(tmp.c_str()) ; // Ѵ
//       out.push_back( atoi(tmp.c_str()) ); // Ѵ
      unsigned int a = tmp.find(" ");
      
      if(a==string::npos){	// 4İʲǺǸǤˤɤĤ
	 break;
      }
      else{
	 tmp.erase(0, a+1);	// ɤ߽äʬ
      }
   }
   return 0;
}


int Linbhand_base::Set(char * motor, char * parameter, int value)
{
   sprintf(m_writebuf, "%sFSET %s %d\r", motor, parameter, value);
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);
   return 0;
}


int Linbhand_base::Command(char * send, char * receive)
{
   sprintf(m_writebuf, "%s\r", send); // command + next line
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(receive);
   return 0;
}


int Linbhand_base::StopMotor(char * motor)
{
   sprintf(m_writebuf, "%sT\r", motor);
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);
   return 0;
}


// RealTime Mode functions

int Linbhand_base::RTAbort()
{
   char a = 0x03;
   sprintf(m_writebuf, "%c", a); // Control-c
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);
   return 0;
}


int Linbhand_base::RTSetFlags( char * motor, bool LCV, int LCVC, bool LCPG, bool LFV, int LFVC, bool LFS, bool LFAP, bool LFDP, int LFDPC )
{
   int tmp[4];
   motorchar2array(motor, tmp);

   for(int i=0;i<4;i++){
      if( tmp[i] == 0 ) continue;
      m_RTflags[i][0] = (int) LCV;
      m_RTflags[i][1] = (int) LCPG;
      m_RTflags[i][2] = (int) LFV;
      m_RTflags[i][3] = (int) LFS;
      m_RTflags[i][4] = (int) LFAP;
      m_RTflags[i][5] = (int) LFDP;
   }

   sprintf( m_writebuf, 
	    "%sFSET LCV %d LCVC %d LCPG %d LFV %d LFVC %d LFS %d LFAP %d LFDP %d LFDPC %d\r",
	    motor, LCV, LCVC, LCPG, LFV, LFVC, LFS, LFAP, LFDP, LFDPC );
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);
   return 0;
//    sprintf(chbuf, "123FSET LCV 1 LCVC 2 LCPG 0 LFV 0 LFVC 1 LFS 0 LFAP 1 LFDP 0 LFDPC 1\r");
}


int Linbhand_base::RTStart(char * motor)
{
   motorchar2array(motor, m_RTmotors);

   sprintf( m_writebuf, "%sLOOP\r", motor);
   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);

   return 0;
}


int Linbhand_base::RTSetVelocity(char motor, int velocity)
{
   if(velocity > 127){
      velocity = 127;
      cerr << "invalid velocity value is set\n"
	   << "(-127 < velocity < 127)\n";
   }
   else if(velocity < -127){
      velocity = -127;
      cerr << "invalid velocity value is set\n"
	   << "(-127 < velocity < 127)\n";
   }

   m_RTvel[ get_motornum(motor) ] = (char) velocity;   

   return 0;
}


int Linbhand_base::RTSetGain(char motor, int gain)
{
   if(gain > 255){
      gain = 255;
      cerr << "invalid gain value is set\n"
	   << "(0 =< gain < 256)\n";
   }
   else if(gain < 0){
      gain = 0;
      cerr << "invalid gain value is set\n"
	   << "(0 =< gain < 256)\n";
   }

   m_RTgain[ get_motornum(motor) ] = (char) gain;   

   return 0;
}


int Linbhand_base::RTUpdate(bool control, bool feedback)
{
   // C, c, A, aΰ̣user manual p.32
   if( control == 1 && feedback == 1){
      m_writebuf[0] = 'C';
   }
   else if( control == 0 && feedback == 1){
      m_writebuf[0] = 'A';
   }
   else if( control == 1 && feedback == 0){
      m_writebuf[0] = 'c';
   }
   else{
      m_writebuf[0] = 'a';
   }

   // 2ʸܰʹߤꤷ®٤ʤɤ
   int j = 1;
   for(int i=0;i<4;i++){
      if(m_RTmotors[i] == 0) continue;
      if(m_RTflags[i][0] == 1){
	 m_writebuf[j] = m_RTvel[i];
	 j++;
      }
      if(m_RTflags[i][1] == 1){
	 m_writebuf[j] = m_RTgain[i];
	 j++;
      }
   }
   m_writebuf[j] = 0;

   string a = m_writebuf;	// Ȥꤢβ
   memcpy( m_writebuf, a.c_str(), a.size() );

   m_scom->CommWrite(m_writebuf);
   m_scom->CommRead(m_readbuf);
   return 0;
}


int Linbhand_base::RTUpdate()
{
   return RTUpdate(1,1);
}


char Linbhand_base::RTGetVelocity(char motor)
{
   char out = 0;
   int m = get_motornum(motor);
   if( m_RTmotors[m] == 0 ){
      cerr << "wrong motor " << motor << " is specified\n";
      return 0;
   }
   if( m_RTflags[m][2] != 1 ){
      cerr << "feedback block is not configured to return the feedback velocity of motor " << motor << "\n";
      return 0;
   }

   int j=0;
   for(int i=0;i<4;i++){
      if( m_RTflags[i][2] == 1 ){
	 if( i == m ) break; // LFV
	 j++;
      }
      if( m_RTflags[i][3] == 1 ) j++; // LFS
      if( m_RTflags[i][4] == 1 ) j += 2; // LFAP, 2 unsigned bytes
      if( m_RTflags[i][5] == 1 ) j++;	 // LFDP
   }
   out = m_readbuf[j];

   return out;
}


int Linbhand_base::RTGetPosition(char motor)
{
   unsigned int out = 0;
   // LFAPintѴΤ˻Ȥ
   union myreg2byte{
      unsigned char R8[2];
      unsigned short int R16;
   } tmp;

   int m = get_motornum(motor);
   if( m_RTmotors[m] == 0 ){
      cerr << "wrong motor " << motor << " is specified\n";
      return 0;
   }
   if( m_RTflags[m][4] != 1 ){
      cerr << "feedback block is not configured to return the absolute position of motor " << motor << "\n";
      return 0;
   }

   // äƤ륹åȤõ
   int j=0;
   for(int i=0;i<4;i++){
      if( m_RTflags[i][2] == 1 ) j++; // LFV
      if( m_RTflags[i][3] == 1 ) j++; // LFS

      if( m_RTflags[i][4] == 1 ){
	 if( i == m ) break; // positionLFAPäƤΤ+2break	 
	 j += 2; // LFAP, 2 unsigned bytes
      }
//       if( i == m ) break; // positionLFAPäƤΤ+2break
//       }
//       if( m_RTflags[i][4] == 1 ) j += 2; // LFAP, 2 unsigned bytes
      if( m_RTflags[i][5] == 1 ) j++;	 // LFDP
   }

   tmp.R8[0] = m_readbuf[j+1];	// ǥ
   tmp.R8[1] = m_readbuf[j];	// դˤʤ
   out = (int) tmp.R16;

   return out;
}


void Linbhand_base::motorchar2array(char * in, int * out)
{
   // ⡼ɽʸflagѴ
   // in: ⡼, out: flag[4]
   // example: in = "123" ==> out[3]Τ0ǤȤ1
   string s = in;
   for(int i=0;i<4;i++) out[i] = 0;

   if( s.find("1") != string::npos ) out[0] = 1;
   if( s.find("2") != string::npos ) out[1] = 1;
   if( s.find("3") != string::npos ) out[2] = 1;
   if( s.find("4") != string::npos || s.find("S") != string::npos ) out[3] = 1;

   if( s.find("G") != string::npos ){
      for(int i=0;i<3;i++) out[i] = 1;
   }

   if( s == "" ){
      for(int i=0;i<4;i++) out[i] = 1;
   }
}


int Linbhand_base::get_motornum(char motor)
{
   int out = 0;
   if(motor == '1') out = 0;
   else if(motor == '2') out = 1;
   else if(motor == '3') out = 2;
   else if(motor == '4' || motor == 'S' ) out = 3;
   else{
      cerr << "invalid motor character\n";
      exit(1);
   }
   return out;
}

