// 
// Linbhand base functions
// BarrettHand C++-function Library for Linux
// 
// Time-stamp: "2007-05-17 18:39:27 sugiura"
// 
// Copyright (c) 2007, Komei Sugiura (komei.sugiura@gmail.com)
// All rights reserved.
// Distributed under the BSD License.
// (See accompanying file license.txt)
// 
// Inspired by Stephen Nestinger's chbhand
// 
// For more detail on serial communication commands, 
// refer to Grasper Control Language Protocol
// www.barretttechnology.com/robot/products/hand/D0AA7A.pdf
// 
// Any comments or suggestions are welcome.


#ifndef Linbhand_base_h
#define Linbhand_base_h

#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <string>
#include <cstring>
#include <vector>

#include "GCLserial.h"

using namespace std;


class Linbhand_base
{
public:
   Linbhand_base::Linbhand_base(string dev);
   virtual Linbhand_base::~Linbhand_base();
public:
   // Supervisory Mode functions
   virtual int Linbhand_base::InitHand(char * motor);   
   virtual int Linbhand_base::Open(char * motor);
   virtual int Linbhand_base::Close(char * motor);
   virtual int Linbhand_base::StepOpen(char * motor, int value);
   virtual int Linbhand_base::StepClose(char * motor, int value);
   virtual int Linbhand_base::GoToDefault(char * motor);
   virtual int Linbhand_base::GoToHome();
   virtual int Linbhand_base::GoToPosition(char * motor, int value);   
   virtual int Linbhand_base::GoToDifferentPositions(int value1, int value2, int value3, int value4);
   virtual int Linbhand_base::Get(char * motor, char * parameter, int * result);   
   virtual int Linbhand_base::Set(char * motor, char * parameter, int value);
   virtual int Linbhand_base::Command(char * send, char * receive);
   virtual int Linbhand_base::StopMotor(char * motor);

public:   
   // RealTime Mode functions
   virtual int Linbhand_base::RTAbort();
   virtual int Linbhand_base::RTSetFlags( char * motor, bool LCV, int LCVC, bool LCPG, bool LFV, int LFVC, bool LFS, bool LFAP, bool LFDP, int LFDPC );
   virtual int Linbhand_base::RTStart(char * motor);
   virtual int Linbhand_base::RTSetVelocity(char motor, int velocity);
   virtual int Linbhand_base::RTSetGain(char motor, int gain);
   virtual int Linbhand_base::RTUpdate(bool control, bool feedback);
   virtual int Linbhand_base::RTUpdate();
   virtual char Linbhand_base::RTGetVelocity(char motor);
   virtual int Linbhand_base::RTGetPosition(char motor);

   
private:
   GCLserial * m_scom;

   string m_dev;
   char m_readbuf[1000];
   char m_writebuf[1000];

   int m_RTmotors[4];		// motorenableˤ뤫
   int m_RTflags[4][6];		// LCV, LCPG, LFV, LFS, LFAP, LFDP parameters for each motor
   char m_RTvel[4];		// loop control velocity (see manual)
   unsigned char m_RTgain[4];	// loop control proportional gain

private:
   void Linbhand_base::motorchar2array(char * in, int * out);
   int Linbhand_base::get_motornum(char motor);
};



#endif
